!----------------------------------------------------------------------------------------------------------
! Recall Version Resource Information Class
! by Brian McGinnis - 2003
! bd@brianmcginnis.com
! Feel free to distribute and modify, no rights reserved.
! Read the article at www.clarionmag.com
!----------------------------------------------------------------------------------------------------------

  MEMBER()
  MAP
    MODULE('API')
      bdGetFileVersionInfoSize(LONG,LONG),LONG,PASCAL,RAW,NAME('GetfileVersionInfoSizeA')
      bdGetFileVersionInfo(LONG,LONG,LONG,LONG),BOOL,PASCAL,RAW,NAME('GetFileVersionInfoA')
      bdVerQueryValue(LONG,LONG,LONG,LONG),BOOL,PASCAL,RAW,NAME('VerQueryValueA')
    END
    MODULE('C')
      bdmemcpy(LONG,LONG,LONG),NAME('_memcpy'),DLL(_ABCDllMode_)
    END

  END

  INCLUDE('BDVER.INC')

!----------------------------------------------------------------------------------------------------------
BDVersionInfo.Construct             PROCEDURE()
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN

!----------------------------------------------------------------------------------------------------------
BDVersionInfo.Destruct              PROCEDURE()
!----------------------------------------------------------------------------------------------------------
  CODE
  IF NOT (SELF.Buffer &= NULL)
    DISPOSE(SELF.Buffer)
  END
  RETURN

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetCodePageID         PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN(SELF.CodePageID)

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetComments           PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('Comments')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetCompanyName        PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('CompanyName')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetFileDescription    PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('FileDescription')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetFilePathName       PROCEDURE() !,STRING,VIRTUAL
! Returns full pathname of file whose resource info is being
! retrieved.
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.FN

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetFileVersion        PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
V       STRING(50)
VLen    LONG,AUTO
I       LONG,AUTO
  CODE
  V = SELF.GetInfo('FileVersion')
  !Replace commas with periods
  VLen = LEN(CLIP(V))
  LOOP I = 1 TO VLen
    IF V[I] = ',' THEN V[I] = '.'.
  END
  RETURN CLIP(V)
      
!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetInfo               PROCEDURE(STRING xStrName)      !,STRING
! Returns value for a given version string.
! Returns string containing value for matching string, empty if not found
!----------------------------------------------------------------------------------------------------------
SubBlock        CSTRING(128)
VersionPtr      LONG
VersionSize     LONG
BufInfo         BYTE,DIM(1024)
RVal            CSTRING(128)

  CODE
  IF xStrName = '' THEN RETURN ''.
  IF ~SELF.Inited 
    IF SELF.Init() THEN RETURN TRUE.
  END

  SubBlock = '\StringFileInfo\' & SELF.LangID & SELF.CodePageID & '\' & CLIP(xStrName)
  VersionPtr = ADDRESS(BufInfo)

  IF bdVerQueryValue(ADDRESS(SELF.Buffer),ADDRESS(SubBlock),ADDRESS(VersionPtr),ADDRESS(VersionSize))
    bdMemCpy(ADDRESS(RVal),VersionPtr,128)
  END
  RETURN RVal

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetInternalName       PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('InternalName')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetLangID             PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.LangID

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetLegalCopyright     PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('LegalCopyright')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetLegalTrademarks    PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('LegalTrademarks')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetOriginalFilename   PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('OriginalFilename')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetProductName        PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('ProductName')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetProductVersion     PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('ProductVersion')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetPrivateBuild       PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('PrivateBuild')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetSpecialBuild       PROCEDURE() !,STRING
!----------------------------------------------------------------------------------------------------------
  CODE
  RETURN SELF.GetInfo('SpecialBuild')

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.GetVersionInfoData    PROCEDURE()     !,BYTE,PRIVATE  
! Attempts to fetch version info on file.  Once we have this data, queries can be made against it.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
Dummy       ULONG
  CODE
  !Fetch the size of version info data
  SELF.BufSize = bdGetFileVersionInfoSize(ADDRESS(SELF.FN),ADDRESS(Dummy))
  IF SELF.BufSize <= 0 THEN RETURN TRUE.
  SELF.Buffer &= NEW(CSTRING(SELF.BufSize+1))

  !Fetch version info into SELF.Buffer
  IF ~bdGetFileVersionInfo(ADDRESS(SELF.FN),Dummy,SELF.BufSize,ADDRESS(SELF.Buffer)) THEN RETURN TRUE.

  RETURN FALSE

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.Init                  PROCEDURE()     !,BYTE
! Sets up object with the exe pathname of the currently executing program.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
  CODE
  SELF.FN = COMMAND('0') 
  IF SELF.InitSvr() THEN RETURN TRUE.
  RETURN FALSE


!----------------------------------------------------------------------------------------------------------
bdVersionInfo.Init                  PROCEDURE(STRING xFN)   !,BYTE
! Set up object with a passed in filename.  The filename must contain the full path to the file.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
  CODE
  IF xFN = '' THEN RETURN TRUE.
  SELF.FN = xFN
  IF SELF.InitSvr() THEN RETURN TRUE.
  RETURN FALSE


!----------------------------------------------------------------------------------------------------------
bdVersionInfo.InitSvr               PROCEDURE() !,BYTE,PRIVATE
! Final .init method called by all other init methods
! Attempts to fetch resource data and establish language ID and
! code page IDs if not already specified.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
  CODE
  IF SELF.GetVersionInfoData() THEN RETURN TRUE.
  IF SELF.LangID = '' THEN SELF.SetLangID(bdLang:USEnglish).
  IF SELF.CodePageID = '' THEN SELF.SetCodePageID(bdCPage:Multi).
  SELF.Inited = TRUE
  RETURN FALSE


!----------------------------------------------------------------------------------------------------------
bdVersionInfo.SetCodePageID         PROCEDURE(STRING xID)   !,BYTE,PROC
! Sets code page identifier string up for use when retrieving values.
! Defaults to Multilingual in .Init method.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
SLen        LONG,AUTO

  CODE
  SLen = LEN(CLIP(xID))
  IF SLen <> 4 THEN RETURN TRUE.

  SELF.CodePageID = LOWER(xID)
  RETURN FALSE

!----------------------------------------------------------------------------------------------------------
bdVersionInfo.SetLangID             PROCEDURE(STRING xID)   !,BYTE,PROC
! Sets language identifier string up for use when retrieving values.
! Defaults to U.S. English in .Init method.
! Returns FALSE if successful.
!----------------------------------------------------------------------------------------------------------
SLen        LONG,AUTO

  CODE
  SLen = LEN(CLIP(xID))
  IF SLen <> 4 THEN RETURN TRUE.

  SELF.LangID = LOWER(xID)
  RETURN FALSE
